from WikidataTemplater.DBHandler import DBhandler
from WikidataTemplater.WDhandler import WDhandler
from WikidataTemplater.DataModel import DataModel
from WikidataTemplater.main import Settings
import json
s: Settings = Settings()
FALLBACK_LANGUAGE = s.settings["FALLBACK_LANGUAGE"]
LANGUAGES = s.settings["LANGUAGES"]


class Route():
    """manages the route usually build from the path
    """    
    def __init__(self, path: str = "") -> None:
        self.language: str = ""
        # Filters are the specific variables within the path
        # Their placehiolders inside WD query templates 
        # are noted as <?f#>, where # represents an integer 
        # (1 for the most left filter)
        self.filters: list[str] = []
        self.template: str = ""
        self.query = ""
        self.path: str = path

    def build_route(self, language: str, entity: str, wdid: str = None) -> str:
        """build a route , typically used vor the creation of link uri

        Args:
            language (str): ISO 639-1 two letter code
            entity (str): a subject or object from the data model
            wdid (str, optional): shortcut Wikidata object identifier (e.g. 'Q42'). Defaults to None.

        Returns:
            str: the newly created route 
        """        
        path: str = self.path
        dtm: DataModel = DataModel()
        
        if dtm.is_singular(entity):
            path += "/" + entity + ";" + self.get_label_encoded(wdid, language) + ";" + wdid
            path += "/" + language
        else:
            path += "/" + dtm.singular_of(entity) + ";" + self.get_label_encoded(wdid, language) + ";" + wdid
            path += "/" + language
        return path
        

    def clear_routes(self) -> None:
        """ removes all records from the "route" table
        """        
        dbh: DBhandler = DBhandler()
        q: str = "delete from routes"
        dbh.delete(q)

    def is_query(self,param: int | str) -> int | None:
        """ lookup for a stored query for a given route or id

        Args:
            param (int | str): the id of the query inside 
            database <templates> table

        Returns:
            int | None: the id of the template or None
        """        
        if param.isnumeric():
            q: str = "SELECT id from templates where id =" + str(param)
        else:
            q: str = "SELECT id from templates where human_readable_name = '" + str(param) + "'"
        dbh: DBhandler = DBhandler()
        r: list[dict] = dbh.get(q)
        if r:
            return r[0]
        else:
            return None
    
    def get_label_encoded(self, wdid: str, language:str = FALLBACK_LANGUAGE) -> str:
        """retrieves the stored encoded value of 
           an Wikidata object label in a given language.
           'encoded' means small letters an spaces replaced by underscores.
           The 'encoded' label will be used within the route

        Args:
            wdid (str): shortcut Wikidata id (e.g. 'Q42')
            language (str, optional): ist 639-1 two letter code. Defaults to FALLBACK_LANGUAGE.

        Returns:
            str: the encoded label
        """        
        dbh: DBhandler = DBhandler()
        q: str = "SELECT label_encoded from routes where wdid = '" + wdid + "' and language = '" + language + "'"
        r: list[dict] = dbh.get(q)
        if r:
            return r[0]
        else:
            return ""

    def get_same_as_template(self, subject: str) -> str | None:
        """looks for the similaire template name within the datamodel

        Args:
            subject (str): original template human readable name

        Returns:
            str: similaire template human redable name
        """        
        dtm = DataModel()
        r = dtm.select(("?" + subject, "?same_template_as", None))
        if r:
            return r[0][2][1:]
        else:
            return None
    


    def get_template(self, param: str) -> None:
        """ retrieves the stored template by a given route
            writes into self.template
        Args:
            param (str): a route string
        """        
        q: str = "SELECT template, title, description, cached from templates where human_readable_name = '" + str(param) + "'"
        dbh: DBhandler = DBhandler()
        r: list[dict]= dbh.get(q)
        if r:
            self.template =  r[0]
            self.title = r[1]
            self.description = r[2]
            self.cached = r[3]
        else:
            self.template = None
            self.title = None
            self.description = None
            self.cached = None
        return self.cached
    
    def parse_route(self,path: str) :
        """parses a given route path
           determines filters, language and the shortened route

        Args:
            path (str): the given route path

        Returns:
            str: the query template, the filter list, the language
        """        
        
        a: list[str] = path.split("/")
        if a[-1:][0].lower() in LANGUAGES:
            self.language = a[-1:][0].lower()
            a = a[0:-1]
        else:
            self.language = FALLBACK_LANGUAGE
        self.path = "/" + "/".join(a)
        self.filters = []
        human_readable_name: str = ""
        for e  in a:
            aa: list[str] = e.split(";")
            human_readable_name += "/" + aa[0]
            if len(aa) > 1:
                if aa[1][0].lower() == "q" and int(aa[1][1:]) > 0:
                    self.filters.append(aa[1])
                elif aa[2][0].lower() == "q" and int(aa[2][1:]) > 0:
                    self.filters.append(aa[2])
        


        if self.is_query(human_readable_name):
            cached = self.get_template(human_readable_name)
            if self.template[0] == "/":
                cached = self.get_template(self.template)
        else:
            same_as_template: str = self.get_same_as_template(human_readable_name)
            cached = self.get_template(same_as_template)

        if self.title:
            title = json.loads(self.title)[self.language]
        else:
            title = "Rebuilding"

        if self.description:
            description = json.loads(self.description)[self.language]
        else:
            description = "…"
        
        return self.template, self.filters, self.language, title, description, cached


    
    def to_title(self, route):
        ''' convert a route to a human redable title'''
        # fixme: needs to work
        return route
     
