
from SPARQLWrapper import SPARQLWrapper
import json
from WikidataTemplater.main import Settings
import typing 

s = Settings()
WIKIDATA_ENDPOINT= s.settings["WIKIDATA_ENDPOINT"]
FALLBACK_LANGUAGE = s.settings["FALLBACK_LANGUAGE"]

class WDhandler():
    """manages queries to Wikidata endpoint

    Returns:
        class: 
    """    
    def __init__(self):
        """class initialization
        """        
        self.sparql = SPARQLWrapper(WIKIDATA_ENDPOINT)
        

    def get(self, query: str) -> dict | None:
        """executes a SPARQL query against the endpoint

        Args:
            str (query): a SPARQL query string

        Returns:
            dict | None: returned json dict from the endpoint or None
        """        
        #fixme Error handling
        #print("SPARQL", query)
        try:
            self.sparql.setQuery(query)
            self.sparql.setReturnFormat('json')
            return self.sparql.query().convert()
        except Exception as err:
            print(f"Unexpected {err=}, {type(err)=}")

    def get_label(self,  wdid: str, language: str) -> str:
        """gets the itemLabel of an WD object in the given language

        Args:
            str (wdid): the shortcut Wikidata id (e.g. 'Q42')
            str (language): ISO-639-1 two-letter-code of the language

        Returns:
            str: the value of the itemLabel
        """        
        # fixme: Error handling
        # fixme: use self.get for execution
        str: q = '''PREFIX rdfs: <http://www.w3.org/2000/01/rdf-schema#> 
            PREFIX wd: <http://www.wikidata.org/entity/> 
            SELECT  *
            WHERE {
                    wd:'''+ wdid + ''' rdfs:label ?label .
                    FILTER (
                        langMatches( lang(?label), "''' + language + '''" ) || 
                        langMatches( lang(?label), "''' + FALLBACK_LANGUAGE + '''" )
                    )
                } 
            LIMIT 2'''
        self.sparql.setQuery(q)
        self.sparql.setReturnFormat("JSON")
        r = self.sparql.query().convert()
        r = r['results']['bindings'].copy()
        if len(r) == 0:
            return ""
        elif len(r) == 1:
            return r[0]['label']['value']
        else:
            for e in r:
                if e['label']['xml:lang'] == language.lower():
                    return e['label']['value']
                
    def get_instance(self, wdid: str) -> str:
        """get the lowest instance of an WD object 
           or instance human (Q5) if part of returned values

        Args:
            str (wdid): the sortcut Wikidata ID (e.g. 'Q42')

        Returns:
            str: shortcut WDID of the lowest instance or empty string
        """        
        # fixme: Error handling
        # fixme: use self.get()
        q = '''SELECT ?instance WHERE {
                wd:''' + wdid + ''' wdt:P31 ?instance .
            }'''

        self.sparql.setQuery(q)
        self.sparql.setReturnFormat("JSON")
        try:
            r = self.sparql.query().convert()
            r = [int(x["instance"]["value"].split("/")[-1:][0].split("Q")[1]) for x in r['results']['bindings']]
            if "5" in r:
                return "Q5"
            else: 
                return "Q" + str(min(r))
        except: 
            return ""

        
        
    def Xget_lowest_instance(self,  a) -> str:
        """Returns the lowest int of an array and 
        returns a string with 'Q' on first position.
        If the is an element 5, the instance Q5 will be returned instead


        Args:
            a (list): list of integer with stripped instances (P31)

        Returns:
            str: shortcut label of the lowest instance
        """   
        # fixme: maybe removable
        if "Q5" in a:
            return "Q5"
        else: 
            return "Q" + str(min(a))

    


